@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -24,8 +24,8 @@ this module can be modified.
 from __future__ import absolute_import, division, print_function
 
 import contextlib
+import datetime
 import inspect
-import os
 import threading
 import warnings
 from enum import Enum, IntEnum, unique
@@ -37,9 +37,9 @@ from hypothesis.errors import (
     InvalidArgument,
     InvalidState,
 )
-from hypothesis.internal.compat import text_type
+from hypothesis.internal.compat import integer_types, quiet_raise, string_types
 from hypothesis.internal.reflection import get_pretty_function_description, proxies
-from hypothesis.internal.validation import try_convert
+from hypothesis.internal.validation import check_type, try_convert
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
@@ -158,22 +158,13 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
 
     def __init__(self, parent=None, **kwargs):
         # type: (settings, **Any) -> None
-        if (
-            kwargs.get("database", not_set) is not_set
-            and kwargs.get("database_file", not_set) is not not_set
-        ):
-            if kwargs["database_file"] is None:
-                kwargs["database"] = None
-            else:
-                from hypothesis.database import ExampleDatabase
-
-                kwargs["database"] = ExampleDatabase(kwargs["database_file"])
-        if not kwargs.get("perform_health_check", True):
-            kwargs["suppress_health_check"] = HealthCheck.all()
-        if kwargs.get("max_shrinks") == 0:
-            kwargs["phases"] = tuple(
-                p for p in _validate_phases(kwargs.get("phases")) if p != Phase.shrink
-            )
+        if kwargs.get("derandomize"):
+            if kwargs.get("database") is not None:
+                raise InvalidArgument(
+                    "derandomize=True implies database=None, so passing "
+                    "database=%r too is invalid." % (kwargs["database"],)
+                )
+            kwargs["database"] = None
         self._construction_complete = False
         deprecations = []
         defaults = parent or settings.default
@@ -182,11 +173,10 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
                 if kwargs.get(setting.name, not_set) is not_set:
                     kwargs[setting.name] = getattr(defaults, setting.name)
                 else:
-                    if kwargs[setting.name] != setting.future_default:
-                        if setting.deprecation_message is not None:
-                            deprecations.append(setting)
                     if setting.validator:
                         kwargs[setting.name] = setting.validator(kwargs[setting.name])
+                    if setting.deprecation_message is not None:
+                        deprecations.append(setting)
         for name, value in kwargs.items():
             if name not in all_settings:
                 raise InvalidArgument(
@@ -197,14 +187,11 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         self._construction_complete = True
 
         for d in deprecations:
-            note_deprecation(d.deprecation_message, self)
-
-    def defaults_stack(self):
-        try:
-            return self.storage.defaults_stack
-        except AttributeError:
-            self.storage.defaults_stack = []
-            return self.storage.defaults_stack
+            note_deprecation(
+                d.deprecation_message,
+                since=d.deprecated_since,
+                verbosity=self.verbosity,
+            )
 
     def __call__(self, test):
         """Make the settings object (self) an attribute of the test.
@@ -247,10 +234,9 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
                     "functions, or on subclasses of GenericStateMachine."
                 )
         if hasattr(test, "_hypothesis_internal_settings_applied"):
-            note_deprecation(
-                "%s has already been decorated with a settings object, which "
-                "will be overridden.  This will be an error in a future "
-                "version of Hypothesis.\n    Previous:  %r\n    This:  %r"
+            raise InvalidArgument(
+                "%s has already been decorated with a settings object."
+                "\n    Previous:  %r\n    This:  %r"
                 % (
                     get_pretty_function_description(test),
                     test._hypothesis_internal_use_settings,
@@ -265,11 +251,9 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
 
         @proxies(test)
         def new_test(*args, **kwargs):
-            note_deprecation(
-                "Using `@settings` without `@given` does not make sense and "
-                "will be an error in a future version of Hypothesis."
+            raise InvalidArgument(
+                "Using `@settings` on a test without `@given` is completely pointless."
             )
-            test(*args, **kwargs)
 
         # @given will get the test from this attribution (rather than use the
         # version with the deprecation warning)
@@ -294,9 +278,8 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         options=None,
         validator=None,
         show_default=True,
-        future_default=not_set,
         deprecation_message=None,
-        hide_repr=not_set,
+        deprecated_since=None,
     ):
         """Add a new setting.
 
@@ -314,22 +297,17 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         if options is not None:
             options = tuple(options)
             assert default in options
-
-        if future_default is not_set:
-            future_default = default
-
-        if hide_repr is not_set:
-            hide_repr = bool(deprecation_message)
+        else:
+            assert validator is not None
 
         all_settings[name] = Setting(
-            name,
-            description.strip(),
-            default,
-            options,
-            validator,
-            future_default,
-            deprecation_message,
-            hide_repr,
+            name=name,
+            description=description.strip(),
+            default=default,
+            options=options,
+            validator=validator,
+            deprecation_message=deprecation_message,
+            deprecated_since=deprecated_since,
         )
         setattr(settings, name, settingsProperty(name, show_default))
 
@@ -363,7 +341,7 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
             value = getattr(self, name)
             # The only settings that are not shown are those that are
             # deprecated and left at their default values.
-            if value != setting.default or not setting.hide_repr:
+            if value != setting.default or not setting.deprecation_message:
                 bits.append("%s=%r" % (name, value))
         return "settings(%s)" % ", ".join(sorted(bits))
 
@@ -375,20 +353,6 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
                 bits.append("%s=%r" % (name, value))
         return ", ".join(sorted(bits, key=len))
 
-    def __enter__(self):
-        note_deprecation(
-            "Settings should be determined only by global state or with the "
-            "@settings decorator."
-        )
-        default_context_manager = default_variable.with_value(self)
-        self.defaults_stack().append(default_context_manager)
-        default_context_manager.__enter__()
-        return self
-
-    def __exit__(self, *args, **kwargs):
-        default_context_manager = self.defaults_stack().pop()
-        return default_context_manager.__exit__(*args, **kwargs)
-
     @staticmethod
     def register_profile(name, parent=None, **kwargs):
         # type: (str, settings, **Any) -> None
@@ -404,27 +368,14 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         keyword arguments for each setting that will be set differently to
         parent (or settings.default, if parent is None).
         """
-        if not isinstance(name, (str, text_type)):
-            note_deprecation("name=%r must be a string" % (name,))
-        if "settings" in kwargs:
-            if parent is None:
-                parent = kwargs.pop("settings")
-                note_deprecation(
-                    "The `settings` argument is deprecated - " "use `parent` instead."
-                )
-            else:
-                raise InvalidArgument(
-                    "The `settings` argument is deprecated, and has been "
-                    "replaced by the `parent` argument.  Use `parent` only."
-                )
+        check_type(string_types, name, "name")
         settings._profiles[name] = settings(parent=parent, **kwargs)
 
     @staticmethod
     def get_profile(name):
         # type: (str) -> settings
         """Return the profile with the given name."""
-        if not isinstance(name, (str, text_type)):
-            note_deprecation("name=%r must be a string" % (name,))
+        check_type(string_types, name, "name")
         try:
             return settings._profiles[name]
         except KeyError:
@@ -439,8 +390,7 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         Any setting not defined in the profile will be the library
         defined default for that setting.
         """
-        if not isinstance(name, (str, text_type)):
-            note_deprecation("name=%r must be a string" % (name,))
+        check_type(string_types, name, "name")
         settings._current_profile = name
         settings._assign_default_internal(settings.get_profile(name))
 
@@ -459,101 +409,81 @@ class Setting(object):
     default = attr.ib()
     options = attr.ib()
     validator = attr.ib()
-    future_default = attr.ib()
     deprecation_message = attr.ib()
-    hide_repr = attr.ib()
+    deprecated_since = attr.ib()
 
 
-settings._define_setting(
-    "min_satisfying_examples",
-    default=not_set,
-    description="""
-This doesn't actually do anything, but remains for compatibility reasons.
-""",
-    deprecation_message="""
-The min_satisfying_examples setting has been deprecated and disabled, due to
-overlap with the filter_too_much healthcheck and poor interaction with the
-max_examples setting.
-""",
-)
+def _ensure_positive_int(x, name, since, min_value=0):
+    if not isinstance(x, integer_types):
+        note_deprecation(
+            "Passing non-integer %s=%r is deprecated" % (name, x), since=since
+        )
+    x = try_convert(int, x, name)
+    if x < min_value:
+        raise InvalidArgument("%s=%r must be at least %r." % (name, x, min_value))
+    return x
+
+
+def _max_examples_validator(x):
+    x = _ensure_positive_int(x, "max_examples", since="2019-03-06", min_value=0)
+    if x == 0:
+        note_deprecation(
+            "max_examples=%r should be at least one. You can disable example "
+            "generation with the `phases` setting instead." % (x,),
+            since="2019-03-06",
+        )
+    return x
+
 
 settings._define_setting(
     "max_examples",
     default=100,
+    validator=_max_examples_validator,
     description="""
 Once this many satisfying examples have been considered without finding any
 counter-example, falsification will terminate.
-""",
-)
 
-settings._define_setting(
-    "max_iterations",
-    default=not_set,
-    description="""
-This doesn't actually do anything, but remains for compatibility reasons.
-""",
-    deprecation_message="""
-The max_iterations setting has been disabled, as internal heuristics are more
-useful for this purpose than a user setting.  It no longer has any effect.
+The default value is chosen to suit a workflow where the test will be part of
+a suite that is regularly executed locally or on a CI server, balancing total
+running time against the chance of missing a bug.
+
+If you are writing one-off tests, running tens of thousands of examples is
+quite reasonable as Hypothesis may miss uncommon bugs with default settings.
+For very complex code, we have observed Hypothesis finding novel bugs after
+*several million* examples while testing :pypi:`SymPy`.
 """,
 )
 
 settings._define_setting(
     "buffer_size",
-    default=8 * 1024,
-    description="""
-The size of the underlying data used to generate examples. If you need to
-generate really large examples you may want to increase this, but it will make
-your tests slower.
-""",
-)
-
-
-settings._define_setting(
-    "max_shrinks",
     default=not_set,
-    description="""
-Passing ``max_shrinks=0`` disables the shrinking phase (see the ``phases``
-setting), but any other value has no effect and uses a general heuristic.
-""",
-    deprecation_message="""
-The max_shrinks setting has been disabled, as internal heuristics are more
-useful for this purpose than a user setting.
-""",
+    validator=lambda x: _ensure_positive_int(x, "buffer_size", since="2019-03-06"),
+    description="The buffer_size setting has been deprecated and no longer does anything.",
+    deprecation_message="The buffer_size setting can safely be removed with no effect.",
+    deprecated_since="2019-07-03",
 )
 
 
-def _validate_timeout(n):
-    if n is unlimited:
-        return -1
-    else:
-        return n
-
-
 settings._define_setting(
     "timeout",
-    default=60,
-    description="""
-Once this many seconds have passed, falsify will terminate even
-if it has not found many examples. This is a soft rather than a hard
-limit - Hypothesis won't e.g. interrupt execution of the called
-function to stop it. If this value is <= 0 then no timeout will be
-applied.
-""",
-    hide_repr=False,  # Still affects behaviour at runtime
-    deprecation_message="""
-The timeout setting is deprecated and will be removed in a future version of
-Hypothesis. To get the future behaviour set ``timeout=hypothesis.unlimited``
-instead (which will remain valid for a further deprecation period after this
-setting has gone away).
-""",
-    future_default=unlimited,
-    validator=_validate_timeout,
+    default=not_set,
+    description="The timeout setting has been deprecated and no longer does anything.",
+    deprecation_message="The timeout setting can safely be removed with no effect.",
+    deprecated_since="2017-11-02",
+    options=(not_set, unlimited),
 )
 
+
+def _derandomize_validator(x):
+    if not isinstance(x, bool):
+        note_deprecation("derandomize=%r should be a bool." % (x,), since="2019-03-06")
+    return bool(x)
+
+
 settings._define_setting(
     "derandomize",
     default=False,
+    validator=_derandomize_validator,
     description="""
 If this is True then hypothesis will run in deterministic mode
 where each falsification uses a random number generator that is seeded
@@ -565,30 +495,12 @@ find novel breakages.
 """,
 )
 
-settings._define_setting(
-    "strict",
-    default=os.getenv("HYPOTHESIS_STRICT_MODE") == "true",
-    description="""
-Strict mode has been deprecated in favor of Python's standard warnings
-controls.  Ironically, enabling it is therefore an error - it only exists so
-that users get the right *type* of error!
-""",
-    deprecation_message="""
-Strict mode is deprecated and will go away in a future version of Hypothesis.
-To get the same behaviour, use
-warnings.simplefilter('error', HypothesisDeprecationWarning).
-""",
-    future_default=False,
-)
 
-
-def _validate_database(db, _from_db_file=False):
+def _validate_database(db):
     from hypothesis.database import ExampleDatabase
 
     if db is None or isinstance(db, ExampleDatabase):
         return db
-    if _from_db_file or db is not_set:
-        return ExampleDatabase(db)
     raise InvalidArgument(
         "Arguments to the database setting must be None or an instance of "
         "ExampleDatabase.  Try passing database=ExampleDatabase(%r), or "
@@ -610,23 +522,6 @@ database, or any path for a directory-based example database.
     validator=_validate_database,
 )
 
-settings._define_setting(
-    "database_file",
-    default=not_set,
-    show_default=False,
-    description="""
-The file or directory location to save and load previously tried examples;
-`:memory:` for an in-memory cache or None to disable caching entirely.
-""",
-    validator=lambda f: _validate_database(f, _from_db_file=True),
-    deprecation_message="""
-The `database_file` setting is deprecated in favor of the `database`
-setting, and will be removed in a future version.  It only exists at
-all for complicated historical reasons and you should just use
-`database` instead.
-""",
-)
-
 
 @unique
 class Phase(IntEnum):
@@ -649,12 +544,8 @@ class HealthCheck(Enum):
     @classmethod
     def all(cls):
         # type: () -> List[HealthCheck]
-        bad = (HealthCheck.exception_in_generation, HealthCheck.random_module)
-        return [h for h in list(cls) if h not in bad]  # type: ignore
-
-    exception_in_generation = 0
-    """Deprecated and no longer does anything. It used to convert errors in
-    data generation into FailedHealthCheck error."""
+        deprecated = [HealthCheck.hung_test]
+        return [x for x in list(HealthCheck) if x not in deprecated]
 
     data_too_large = 1
     """Check for when the typical size of the examples you are generating
@@ -669,23 +560,21 @@ class HealthCheck(Enum):
     """Check for when your data generation is extremely slow and likely to hurt
     testing."""
 
-    random_module = 4
-    """Deprecated and no longer does anything. It used to check for whether
-    your tests used the global random module. Now @given tests automatically
-    seed random so this is no longer an error."""
-
     return_value = 5
     """Checks if your tests return a non-None value (which will be ignored and
     is unlikely to do what you want)."""
 
     hung_test = 6
-    """Checks if your tests have been running for a very long time."""
+    """This health check is deprecated and no longer has any effect.
+    You can use the ``max_examples`` and ``deadline`` settings together to cap
+    the total runtime of your tests, rather than the previous fixed limit."""
 
     large_base_example = 7
     """Checks if the natural example to shrink towards is very large."""
 
     not_a_test_method = 8
-    """Checks if @given has been applied to a method of unittest.TestCase."""
+    """Checks if :func:`@given <hypothesis.given>` has been applied to a
+    method of :class:`python:unittest.TestCase`."""
 
 
 @unique
@@ -702,27 +591,6 @@ class Verbosity(IntEnum):
     verbose = 2
     debug = 3
 
-    @staticmethod
-    def _get_default():
-        # type: () -> Verbosity
-        var = os.getenv("HYPOTHESIS_VERBOSITY_LEVEL")
-        if var is not None:  # pragma: no cover
-            try:
-                result = Verbosity[var]
-            except KeyError:
-                raise InvalidArgument("No such verbosity level %r" % (var,))
-
-            warnings.warn(
-                HypothesisDeprecationWarning(
-                    "The HYPOTHESIS_VERBOSITY_LEVEL environment variable is "
-                    "deprecated, and will be ignored by a future version of "
-                    "Hypothesis.  Configure your verbosity level via a "
-                    "settings profile instead."
-                )
-            )
-            return result
-        return Verbosity.normal
-
     def __repr__(self):
         return "Verbosity.%s" % (self.name,)
 
@@ -730,7 +598,7 @@ class Verbosity(IntEnum):
 settings._define_setting(
     "verbosity",
     options=tuple(Verbosity),
-    default=Verbosity._get_default(),
+    default=Verbosity.normal,
     description="Control the verbosity level of Hypothesis messages",
 )
 
@@ -758,21 +626,21 @@ settings._define_setting(
 settings._define_setting(
     name="stateful_step_count",
     default=50,
+    validator=lambda x: _ensure_positive_int(x, "stateful_step_count", "RELEASEDAY"),
     description="""
 Number of steps to run a stateful program for before giving up on it breaking.
 """,
 )
 
 settings._define_setting(
-    "perform_health_check",
-    default=not_set,
-    description=u"""
-If set to True, Hypothesis will run a preliminary health check before
-attempting to actually execute your test.
-""",
-    deprecation_message="""
-This setting is deprecated, as `perform_health_check=False` duplicates the
-effect of `suppress_health_check=HealthCheck.all()`.  Use that instead!
+    name="report_multiple_bugs",
+    default=True,
+    options=(True, False),
+    description="""
+Because Hypothesis runs the test many times, it can sometimes find multiple
+bugs in a single run.  Reporting all of them at once is usually very useful,
+but replacing the exceptions can occasionally clash with debuggers.
+If disabled, only the exception with the smallest minimal example is raised.
 """,
 )
 
@@ -781,23 +649,15 @@ def validate_health_check_suppressions(suppressions):
     suppressions = try_convert(list, suppressions, "suppress_health_check")
     for s in suppressions:
         if not isinstance(s, HealthCheck):
-            note_deprecation(
-                (
-                    "Non-HealthCheck value %r of type %s in suppress_health_check "
-                    "will be ignored, and will become an error in a future "
-                    "version of Hypothesis"
-                )
+            raise InvalidArgument(
+                "Non-HealthCheck value %r of type %s is invalid in suppress_health_check."
                 % (s, type(s).__name__)
             )
-        elif s in (HealthCheck.exception_in_generation, HealthCheck.random_module):
+        if s is HealthCheck.hung_test:
             note_deprecation(
-                (
-                    "%s is now ignored and suppressing it is a no-op. This will "
-                    "become an error in a future version of Hypothesis. Simply "
-                    "remove it from your list of suppressions to get the same "
-                    "effect."
-                )
-                % (s,)
+                "HealthCheck.hung_test is deprecated and has no "
+                "effect, as we no longer run this health check.",
+                since="2019-01-24",
             )
     return suppressions
 
@@ -809,34 +669,59 @@ settings._define_setting(
     validator=validate_health_check_suppressions,
 )
 
+
+class duration(datetime.timedelta):
+    """A timedelta specifically measured in milliseconds."""
+
+    def __repr__(self):
+        return "timedelta(milliseconds=%r)" % (self.total_seconds() * 1000,)
+
+
+def _validate_deadline(x):
+    if isinstance(x, bool):
+        note_deprecation(
+            "The deadline=%r must be a duration in milliseconds, or None to disable."
+            "  Boolean deadlines are treated as ints, and deprecated." % (x,),
+            since="2019-03-06",
+        )
+    if x is None:
+        return x
+    if isinstance(x, integer_types + (float,)):
+        try:
+            x = duration(milliseconds=x)
+        except OverflowError:
+            quiet_raise(
+                InvalidArgument(
+                    "deadline=%r is invalid, because it is too large to represent "
+                    "as a timedelta. Use deadline=None to disable deadlines." % (x,)
+                )
+            )
+    if isinstance(x, datetime.timedelta):
+        if x <= datetime.timedelta(0):
+            raise InvalidArgument(
+                "deadline=%r is invalid, because it is impossible to meet a "
+                "deadline <= 0. Use deadline=None to disable deadlines." % (x,)
+            )
+        return duration(seconds=x.total_seconds())
+    raise InvalidArgument(
+        "deadline=%r (type %s) must be a timedelta object, an integer or float number of milliseconds, "
+        "or None to disable the per-test-case deadline." % (x, type(x).__name__)
+    )
+
+
 settings._define_setting(
     "deadline",
-    default=not_set,
+    default=duration(milliseconds=200),
+    validator=_validate_deadline,
     description=u"""
-If set, a time in milliseconds (which may be a float to express
-smaller units of time) that each individual example (i.e. each time your test
+If set, a duration (as timedelta, or integer or float number of milliseconds)
+that each individual example (i.e. each time your test
 function is called, not the whole decorated test) within a test is not
 allowed to exceed. Tests which take longer than that may be converted into
 errors (but will not necessarily be if close to the deadline, to allow some
 variability in test run time).
 
 Set this to None to disable this behaviour entirely.
-
-In future this will default to 200. For now, a
-HypothesisDeprecationWarning will be emitted if you exceed that default
-deadline and have not explicitly set a deadline yourself.
-""",
-)
-
-settings._define_setting(
-    "use_coverage",
-    default=not_set,
-    deprecation_message="""
-use_coverage no longer does anything and can be removed from your settings.
-""",
-    description="""
-A flag to enable a feature that no longer exists. This setting is present
-only for backwards compatibility purposes.
 """,
 )
 
@@ -849,15 +734,8 @@ class PrintSettings(Enum):
     """Never print a blob."""
 
     INFER = 1
-    """Make an educated guess as to whether it would be appropriate to print
-    the blob.
-
-    The current rules are that this will print if:
-
-    1. The output from Hypothesis appears to be unsuitable for use with
-       :func:`~hypothesis.example`, and
-    2. The output is not too long, and
-    3. Verbosity is at least normal."""
+    """This option is deprecated and will be treated as equivalent to
+    ALWAYS."""
 
     ALWAYS = 2
     """Always print a blob on failure."""
@@ -867,47 +745,43 @@ class PrintSettings(Enum):
 
 
 def _validate_print_blob(value):
-    if isinstance(value, bool):
-        if value:
-            replacement = PrintSettings.ALWAYS
-        else:
-            replacement = PrintSettings.NEVER
+    if isinstance(value, PrintSettings):
+        replacement = value != PrintSettings.NEVER
 
         note_deprecation(
             "Setting print_blob=%r is deprecated and will become an error "
             "in a future version of Hypothesis. Use print_blob=%r instead."
-            % (value, replacement)
+            % (value, replacement),
+            since="2018-09-30",
         )
         return replacement
 
-    # Values that aren't bool or PrintSettings will be turned into hard errors
-    # by the 'options' check.
+    check_type(bool, value, "print_blob")
+
     return value
 
 
 settings._define_setting(
     "print_blob",
-    default=PrintSettings.INFER,
+    default=False,
     description="""
-Determines whether to print blobs after tests that can be used to reproduce
-failures.
-
-See :ref:`the documentation on @reproduce_failure <reproduce_failure>` for
-more details of this behaviour.
+If set to True, Hypothesis will print code for failing examples that can be used with
+:func:`@reproduce_failure <hypothesis.reproduce_failure>` to reproduce the failing example.
 """,
     validator=_validate_print_blob,
-    options=tuple(PrintSettings),
 )
 
 settings.lock_further_definitions()
 
 
-def note_deprecation(message, s=None):
-    # type: (str, settings) -> None
-    if s is None:
-        s = settings.default
-    assert s is not None
-    verbosity = s.verbosity
+def note_deprecation(message, since, verbosity=None):
+    # type: (str, str, Verbosity) -> None
+    if verbosity is None:
+        verbosity = settings.default.verbosity
+    assert verbosity is not None
+    if since != "RELEASEDAY":
+        date = datetime.datetime.strptime(since, "%Y-%m-%d").date()
+        assert datetime.date(2016, 1, 1) <= date
     warning = HypothesisDeprecationWarning(message)
     if verbosity > Verbosity.quiet:
         warnings.warn(warning, stacklevel=2)
